@@ -213,7 +213,7 @@ gboolean           nautilus_directory_is_local                 (NautilusDirector
 
 gboolean           nautilus_directory_is_in_trash              (NautilusDirectory         *directory);
 gboolean           nautilus_directory_is_in_recent             (NautilusDirectory         *directory);
-gboolean           nautilus_directory_is_in_network            (NautilusDirectory         *directory);
+gboolean           nautilus_directory_is_remote                (NautilusDirectory         *directory);
 
 /* Return false if directory contains anything besides a Nautilus metafile.
  * Only valid if directory is monitored. Used by the Trash monitor.
