@@ -67,14 +67,12 @@ struct _GtkPlacesViewPrivate
   GtkWidget                     *recent_servers_popover;
   GtkWidget                     *recent_servers_stack;
   GtkWidget                     *stack;
-  GtkWidget                     *network_header_spinner;
   GtkWidget                     *network_placeholder;
   GtkWidget                     *network_placeholder_label;
 
   GtkEntryCompletion            *address_entry_completion;
   GtkListStore                  *completion_store;
 
-  GList                         *detected_networks;
   GCancellable                  *networks_fetching_cancellable;
 
   guint                          local_only : 1;
@@ -82,6 +80,8 @@ struct _GtkPlacesViewPrivate
   guint                          should_pulse_entry : 1;
   guint                          entry_pulse_timeout_id;
   guint                          connecting_to_server : 1;
+  guint                          mounting_volume : 1;
+  guint                          unmounting_mount : 1;
   guint                          fetching_networks : 1;
   guint                          loading : 1;
 };
@@ -99,6 +99,16 @@ static gboolean    on_row_popup_menu                             (GtkPlacesViewR
 
 static void        populate_servers                              (GtkPlacesView *view);
 
+static gboolean    gtk_places_view_get_fetching_networks         (GtkPlacesView *view);
+
+static void        gtk_places_view_set_fetching_networks         (GtkPlacesView *view,
+                                                                  gboolean       fetching_networks);
+
+static void        gtk_places_view_set_loading                   (GtkPlacesView *view,
+                                                                  gboolean       loading);
+
+static void        update_loading                                (GtkPlacesView *view);
+
 G_DEFINE_TYPE_WITH_PRIVATE (GtkPlacesView, gtk_places_view, GTK_TYPE_BOX)
 
 /* GtkPlacesView properties & signals */
@@ -106,6 +116,7 @@ enum {
   PROP_0,
   PROP_LOCAL_ONLY,
   PROP_OPEN_FLAGS,
+  PROP_FETCHING_NETWORKS,
   PROP_LOADING,
   LAST_PROP
 };
@@ -415,6 +426,10 @@ gtk_places_view_get_property (GObject    *object,
       g_value_set_boolean (value, gtk_places_view_get_loading (self));
       break;
 
+    case PROP_FETCHING_NETWORKS:
+      g_value_set_boolean (value, gtk_places_view_get_fetching_networks (self));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
@@ -870,14 +885,10 @@ update_network_state (GtkPlacesView *view)
       g_object_set_data (G_OBJECT (priv->network_placeholder),
                          "is-placeholder", GINT_TO_POINTER (TRUE));
       gtk_container_add (GTK_CONTAINER (priv->listbox), priv->network_placeholder);
-
-      gtk_widget_show_all (GTK_WIDGET (priv->network_placeholder));
-      gtk_list_box_invalidate_headers (GTK_LIST_BOX (priv->listbox));
     }
 
-  if (priv->fetching_networks)
+  if (gtk_places_view_get_fetching_networks (view))
     {
-      gtk_spinner_start (GTK_SPINNER (priv->network_header_spinner));
       /* only show a placeholder with a message if the list is empty.
        * otherwise just show the spinner in the header */
       if (!has_networks (view))
@@ -889,24 +900,22 @@ update_network_state (GtkPlacesView *view)
     }
   else if (!has_networks (view))
     {
-      gtk_spinner_stop (GTK_SPINNER (priv->network_header_spinner));
       gtk_widget_show_all (priv->network_placeholder);
       gtk_label_set_text (GTK_LABEL (priv->network_placeholder_label),
-                          _("No networks locations found"));
+                          _("No network locations found"));
     }
   else
     {
-      gtk_spinner_stop (GTK_SPINNER (priv->network_header_spinner));
       gtk_widget_hide (priv->network_placeholder);
     }
 }
 
 static void
 populate_networks (GtkPlacesView   *view,
-                   GFileEnumerator *enumerator)
+                   GFileEnumerator *enumerator,
+                   GList           *detected_networks)
 {
   GList *l;
-  GtkPlacesViewPrivate *priv;
   GFile *file;
   GFile *activatable_file;
   gchar *uri;
@@ -914,10 +923,7 @@ populate_networks (GtkPlacesView   *view,
   GIcon *icon;
   gchar *display_name;
 
-
-  priv = gtk_places_view_get_instance_private (view);
-
-  for (l = priv->detected_networks; l != NULL; l = l->next)
+  for (l = detected_networks; l != NULL; l = l->next)
     {
       file = g_file_enumerator_get_child (enumerator, l->data);
       type = g_file_info_get_file_type (l->data);
@@ -936,8 +942,6 @@ populate_networks (GtkPlacesView   *view,
       g_clear_object (&file);
       g_clear_object (&activatable_file);
     }
-
-  g_clear_object (&enumerator);
 }
 
 static void
@@ -947,37 +951,36 @@ network_enumeration_next_files_finished (GObject      *source_object,
 {
   GtkPlacesViewPrivate *priv;
   GtkPlacesView *view;
+  GList *detected_networks;
   GError *error;
 
   view = GTK_PLACES_VIEW (user_data);
   priv = gtk_places_view_get_instance_private (view);
   error = NULL;
 
-  /* clean previous fetched networks */
-  g_list_free_full (priv->detected_networks, g_object_unref);
-  priv->fetching_networks = FALSE;
+  gtk_places_view_set_fetching_networks (view, FALSE);
+  detected_networks = g_file_enumerator_next_files_finish (G_FILE_ENUMERATOR (source_object),
+                                                           res, &error);
 
-  priv->detected_networks = g_file_enumerator_next_files_finish (G_FILE_ENUMERATOR (source_object),
-                                                                 res, &error);
   if (error)
     {
-      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
         g_warning ("Failed to fetch network locations: %s", error->message);
 
       g_clear_error (&error);
-      g_clear_object (&source_object);
     }
   else
     {
-      populate_networks (view, G_FILE_ENUMERATOR (source_object));
+      populate_networks (view, G_FILE_ENUMERATOR (source_object), detected_networks);
+
+      g_list_free_full (detected_networks, g_object_unref);
     }
 
     /* avoid to update widgets if the operation was cancelled in finalize */
     if (priv->listbox != NULL)
       update_network_state (view);
 
-  priv->loading = FALSE;
-  g_object_notify_by_pspec (G_OBJECT (view), properties[PROP_LOADING]);
+  update_loading (view);
 }
 
 static void
@@ -987,23 +990,22 @@ network_enumeration_finished (GObject      *source_object,
 {
   GtkPlacesViewPrivate *priv;
   GFileEnumerator *enumerator;
-  GtkPlacesView *view;
   GError *error;
 
-  view = GTK_PLACES_VIEW (user_data);
-  priv = gtk_places_view_get_instance_private (view);
   error = NULL;
   enumerator = g_file_enumerate_children_finish (G_FILE (source_object), res, &error);
 
   if (error)
     {
-      if (error->code != G_IO_ERROR_CANCELLED)
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
+          !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED))
         g_warning ("Failed to fetch network locations: %s", error->message);
 
       g_clear_error (&error);
     }
   else
     {
+      priv = gtk_places_view_get_instance_private (GTK_PLACES_VIEW (user_data));
       g_file_enumerator_next_files_async (enumerator,
                                           G_MAXINT32,
                                           G_PRIORITY_DEFAULT,
@@ -1025,8 +1027,7 @@ fetch_networks (GtkPlacesView *view)
   g_cancellable_cancel (priv->networks_fetching_cancellable);
   g_clear_object (&priv->networks_fetching_cancellable);
   priv->networks_fetching_cancellable = g_cancellable_new ();
-  priv->fetching_networks = TRUE;
-
+  gtk_places_view_set_fetching_networks (view, TRUE);
   update_network_state (view);
 
   g_file_enumerate_children_async (network_file,
@@ -1058,10 +1059,8 @@ update_places (GtkPlacesView *view)
   children = gtk_container_get_children (GTK_CONTAINER (priv->listbox));
   g_list_free_full (children, (GDestroyNotify) gtk_widget_destroy);
   priv->network_placeholder = NULL;
-  priv->network_header_spinner = NULL;
-
-  priv->loading = TRUE;
-  g_object_notify_by_pspec (G_OBJECT (view), properties[PROP_LOADING]);
+  /* Inform clients that we started loading */
+  gtk_places_view_set_loading (view, TRUE);
 
   /* Add "Computer" row */
   file = g_file_new_for_path ("/");
@@ -1135,9 +1134,12 @@ update_places (GtkPlacesView *view)
   populate_servers (view);
 
   /* fetch networks and add them asynchronously */
-  fetch_networks (view);
+  if (!gtk_places_view_get_local_only (view))
+    fetch_networks (view);
 
   update_view_mode (view);
+  /* Check whether we still are in a loading state */
+  update_loading (view);
 }
 
 static void
@@ -1151,21 +1153,11 @@ server_mount_ready_cb (GObject      *source_file,
   GError *error;
   GFile *location;
 
-  view = GTK_PLACES_VIEW (user_data);
-  priv = gtk_places_view_get_instance_private (view);
   location = G_FILE (source_file);
   should_show = TRUE;
   error = NULL;
 
-  priv->should_pulse_entry = FALSE;
-  set_busy_cursor (view, FALSE);
-
   g_file_mount_enclosing_volume_finish (location, res, &error);
-  /* Restore from Cancel to Connect */
-  gtk_button_set_label (GTK_BUTTON (priv->connect_button), _("Con_nect"));
-  gtk_widget_set_sensitive (priv->address_entry, TRUE);
-  priv->connecting_to_server = FALSE;
-
   if (error)
     {
       should_show = FALSE;
@@ -1182,14 +1174,30 @@ server_mount_ready_cb (GObject      *source_file,
                (error->code != G_IO_ERROR_CANCELLED &&
                 error->code != G_IO_ERROR_FAILED_HANDLED))
         {
+          view = GTK_PLACES_VIEW (user_data);
           /* if it wasn't cancelled show a dialog */
           emit_show_error_message (view, _("Unable to access location"), error->message);
           should_show = FALSE;
         }
-
+      else
+        {
+          /* it was cancelled, so probably it was called during finalize, bail out. */
+          g_clear_error (&error);
+          return;
+        }
       g_clear_error (&error);
     }
 
+  view = GTK_PLACES_VIEW (user_data);
+  priv = gtk_places_view_get_instance_private (view);
+  priv->should_pulse_entry = FALSE;
+
+  /* Restore from Cancel to Connect */
+  gtk_button_set_label (GTK_BUTTON (priv->connect_button), _("Con_nect"));
+  gtk_widget_set_sensitive (priv->address_entry, TRUE);
+  priv->connecting_to_server = FALSE;
+  update_loading (view);
+
   if (should_show)
     {
       server_list_add_server (view, location);
@@ -1240,14 +1248,10 @@ volume_mount_ready_cb (GObject      *source_volume,
   GVolume *volume;
   GError *error;
 
-  view = GTK_PLACES_VIEW (user_data);
-  priv = gtk_places_view_get_instance_private (view);
   volume = G_VOLUME (source_volume);
   should_show = TRUE;
   error = NULL;
 
-  set_busy_cursor (view, FALSE);
-
   g_volume_mount_finish (volume, res, &error);
 
   if (error)
@@ -1270,10 +1274,21 @@ volume_mount_ready_cb (GObject      *source_volume,
           emit_show_error_message (GTK_PLACES_VIEW (user_data), _("Unable to access location"), error->message);
           should_show = FALSE;
         }
+      else
+        {
+          /* it was cancelled, so probably it was called during finalize, bail out. */
+          g_clear_error (&error);
+          return;
+        }
 
       g_clear_error (&error);
     }
 
+  view = GTK_PLACES_VIEW (user_data);
+  priv = gtk_places_view_get_instance_private (view);
+  priv->mounting_volume = FALSE;
+  update_loading (view);
+
   if (should_show)
     {
       GMount *mount;
@@ -1298,6 +1313,7 @@ unmount_ready_cb (GObject      *source_mount,
                   gpointer      user_data)
 {
   GtkPlacesView *view;
+  GtkPlacesViewPrivate *priv;
   GMount *mount;
   GError *error;
 
@@ -1305,7 +1321,9 @@ unmount_ready_cb (GObject      *source_mount,
   mount = G_MOUNT (source_mount);
   error = NULL;
 
-  set_busy_cursor (view, FALSE);
+  priv = gtk_places_view_get_instance_private (view);
+  priv->unmounting_mount = FALSE;
+  update_loading (view);
 
   g_mount_unmount_with_operation_finish (mount, res, &error);
 
@@ -1358,12 +1376,13 @@ unmount_mount (GtkPlacesView *view,
   toplevel = gtk_widget_get_toplevel (GTK_WIDGET (view));
   operation = gtk_mount_operation_new (GTK_WINDOW (toplevel));
 
-  set_busy_cursor (GTK_PLACES_VIEW (view), TRUE);
-
   g_cancellable_cancel (priv->cancellable);
   g_clear_object (&priv->cancellable);
   priv->cancellable = g_cancellable_new ();
 
+  priv->unmounting_mount = TRUE;
+  update_loading (view);
+
   operation = gtk_mount_operation_new (GTK_WINDOW (toplevel));
   g_mount_unmount_with_operation (mount,
                                   0,
@@ -1394,13 +1413,13 @@ mount_server (GtkPlacesView *view,
   toplevel = gtk_widget_get_toplevel (GTK_WIDGET (view));
   operation = gtk_mount_operation_new (GTK_WINDOW (toplevel));
 
-  set_busy_cursor (view, TRUE);
   priv->should_pulse_entry = TRUE;
   gtk_entry_set_progress_pulse_step (GTK_ENTRY (priv->address_entry), 0.1);
   /* Allow to cancel the operation */
   gtk_button_set_label (GTK_BUTTON (priv->connect_button), _("Cance_l"));
   gtk_widget_set_sensitive (priv->address_entry, FALSE);
   priv->connecting_to_server = TRUE;
+  update_loading (view);
 
   if (priv->entry_pulse_timeout_id == 0)
     priv->entry_pulse_timeout_id = g_timeout_add (100, (GSourceFunc) pulse_entry_cb, view);
@@ -1430,12 +1449,13 @@ mount_volume (GtkPlacesView *view,
   toplevel = gtk_widget_get_toplevel (GTK_WIDGET (view));
   operation = gtk_mount_operation_new (GTK_WINDOW (toplevel));
 
-  set_busy_cursor (view, TRUE);
-
   g_cancellable_cancel (priv->cancellable);
   g_clear_object (&priv->cancellable);
   priv->cancellable = g_cancellable_new ();
 
+  priv->mounting_volume = TRUE;
+  update_loading (view);
+
   g_mount_operation_set_password_save (operation, G_PASSWORD_SAVE_FOR_SESSION);
 
   g_volume_mount (volume,
@@ -1882,11 +1902,13 @@ listbox_filter_func (GtkListBoxRow *row,
   gboolean is_network;
   gboolean is_placeholder;
   gboolean retval;
+  gboolean searching;
   gchar *name;
   gchar *path;
 
   priv = gtk_places_view_get_instance_private (GTK_PLACES_VIEW (user_data));
   retval = FALSE;
+  searching = priv->search_query && priv->search_query[0] != '\0';
 
   is_network = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (row), "is-network"));
   is_placeholder = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (row), "is-placeholder"));
@@ -1894,10 +1916,10 @@ listbox_filter_func (GtkListBoxRow *row,
   if (is_network && priv->local_only)
     return FALSE;
 
-  if (is_placeholder)
+  if (is_placeholder && searching)
     return FALSE;
 
-  if (!priv->search_query || priv->search_query[0] == '\0')
+  if (!searching)
     return TRUE;
 
   g_object_get (row,
@@ -1922,11 +1944,9 @@ listbox_header_func (GtkListBoxRow *row,
                      GtkListBoxRow *before,
                      gpointer       user_data)
 {
-  GtkPlacesViewPrivate *priv;
   gboolean row_is_network;
   gchar *text;
 
-  priv = gtk_places_view_get_instance_private (GTK_PLACES_VIEW (user_data));
   text = NULL;
   row_is_network = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (row), "is-network"));
 
@@ -1964,18 +1984,25 @@ listbox_header_func (GtkListBoxRow *row,
       if (row_is_network)
         {
           GtkWidget *header_name;
+          GtkWidget *network_header_spinner;
 
           g_object_set (label,
                         "margin-end", 6,
                         NULL);
 
           header_name = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-          priv->network_header_spinner = gtk_spinner_new ();
-          g_object_set (priv->network_header_spinner,
+          network_header_spinner = gtk_spinner_new ();
+          g_object_set (network_header_spinner,
                         "margin-end", 12,
                         NULL);
+          g_object_bind_property (GTK_PLACES_VIEW (user_data),
+                                  "fetching-networks",
+                                  network_header_spinner,
+                                  "active",
+                                  G_BINDING_SYNC_CREATE);
+
           gtk_container_add (GTK_CONTAINER (header_name), label);
-          gtk_container_add (GTK_CONTAINER (header_name), priv->network_header_spinner);
+          gtk_container_add (GTK_CONTAINER (header_name), network_header_spinner);
           gtk_container_add (GTK_CONTAINER (header), header_name);
         }
       else
@@ -2007,8 +2034,8 @@ listbox_sort_func (GtkListBoxRow *row1,
 {
   gboolean row1_is_network;
   gboolean row2_is_network;
-  gchar *location1;
-  gchar *location2;
+  gchar *path1;
+  gchar *path2;
   gboolean *is_placeholder1;
   gboolean *is_placeholder2;
   gint retval;
@@ -2032,13 +2059,13 @@ listbox_sort_func (GtkListBoxRow *row1,
   if (is_placeholder2)
     return 1;
 
-  g_object_get (row1, "path", &location1, NULL);
-  g_object_get (row2, "path", &location2, NULL);
+  g_object_get (row1, "path", &path1, NULL);
+  g_object_get (row2, "path", &path2, NULL);
 
-  retval = g_strcmp0 (location1, location2);
+  retval = g_utf8_collate (path1, path2);
 
-  g_free (location1);
-  g_free (location2);
+  g_free (path1);
+  g_free (path2);
 
   return retval;
 }
@@ -2134,7 +2161,7 @@ gtk_places_view_class_init (GtkPlacesViewClass *klass)
    * Since: 3.18
    */
   places_view_signals [OPEN_LOCATION] =
-          g_signal_new (_("open-location"),
+          g_signal_new ("open-location",
                         G_OBJECT_CLASS_TYPE (object_class),
                         G_SIGNAL_RUN_FIRST,
                         G_STRUCT_OFFSET (GtkPlacesViewClass, open_location),
@@ -2157,7 +2184,7 @@ gtk_places_view_class_init (GtkPlacesViewClass *klass)
    * Since: 3.18
    */
   places_view_signals [SHOW_ERROR_MESSAGE] =
-          g_signal_new (_("show-error-message"),
+          g_signal_new ("show-error-message",
                         G_OBJECT_CLASS_TYPE (object_class),
                         G_SIGNAL_RUN_FIRST,
                         G_STRUCT_OFFSET (GtkPlacesViewClass, show_error_message),
@@ -2169,22 +2196,29 @@ gtk_places_view_class_init (GtkPlacesViewClass *klass)
 
   properties[PROP_LOCAL_ONLY] =
           g_param_spec_boolean ("local-only",
-                                _("Local Only"),
-                                _("Whether the sidebar only includes local files"),
+                                "Local Only",
+                                "Whether the sidebar only includes local files",
                                 FALSE,
                                 G_PARAM_READWRITE);
 
   properties[PROP_LOADING] =
           g_param_spec_boolean ("loading",
-                                _("Loading"),
-                                _("Whether the view is loading locations"),
+                                "Loading",
+                                "Whether the view is loading locations",
+                                FALSE,
+                                G_PARAM_READABLE);
+
+  properties[PROP_FETCHING_NETWORKS] =
+          g_param_spec_boolean ("fetching-networks",
+                                "Fetching networks",
+                                "Whether the view is fetching networks",
                                 FALSE,
                                 G_PARAM_READABLE);
 
   properties[PROP_OPEN_FLAGS] =
           g_param_spec_flags ("open-flags",
-                              _("Open Flags"),
-                              _("Modes in which the calling application can open locations selected in the sidebar"),
+                              "Open Flags",
+                              "Modes in which the calling application can open locations selected in the sidebar",
                               GTK_TYPE_PLACES_OPEN_FLAGS,
                               GTK_PLACES_OPEN_NORMAL,
                               G_PARAM_READWRITE);
@@ -2378,6 +2412,68 @@ gtk_places_view_get_loading (GtkPlacesView *view)
   return priv->loading;
 }
 
+static void
+update_loading (GtkPlacesView *view)
+{
+  GtkPlacesViewPrivate *priv;
+  gboolean loading;
+
+  g_return_if_fail (GTK_IS_PLACES_VIEW (view));
+
+  priv = gtk_places_view_get_instance_private (view);
+  loading = priv->fetching_networks || priv->connecting_to_server ||
+            priv->mounting_volume || priv->unmounting_mount;
+
+  set_busy_cursor (view, loading);
+  gtk_places_view_set_loading (view, loading);
+}
+
+static void
+gtk_places_view_set_loading (GtkPlacesView *view,
+                             gboolean       loading)
+{
+  GtkPlacesViewPrivate *priv;
+
+  g_return_if_fail (GTK_IS_PLACES_VIEW (view));
+
+  priv = gtk_places_view_get_instance_private (view);
+
+  if (priv->loading != loading)
+    {
+      priv->loading = loading;
+      g_object_notify_by_pspec (G_OBJECT (view), properties [PROP_LOADING]);
+    }
+}
+
+static gboolean
+gtk_places_view_get_fetching_networks (GtkPlacesView *view)
+{
+  GtkPlacesViewPrivate *priv;
+
+  g_return_val_if_fail (GTK_IS_PLACES_VIEW (view), FALSE);
+
+  priv = gtk_places_view_get_instance_private (view);
+
+  return priv->fetching_networks;
+}
+
+static void
+gtk_places_view_set_fetching_networks (GtkPlacesView *view,
+                                       gboolean       fetching_networks)
+{
+  GtkPlacesViewPrivate *priv;
+
+  g_return_if_fail (GTK_IS_PLACES_VIEW (view));
+
+  priv = gtk_places_view_get_instance_private (view);
+
+  if (priv->fetching_networks != fetching_networks)
+    {
+      priv->fetching_networks = fetching_networks;
+      g_object_notify_by_pspec (G_OBJECT (view), properties [PROP_FETCHING_NETWORKS]);
+    }
+}
+
 /**
  * gtk_places_view_get_local_only:
  * @view: a #GtkPlacesView
@@ -2427,7 +2523,7 @@ gtk_places_view_set_local_only (GtkPlacesView *view,
       priv->local_only = local_only;
 
       gtk_widget_set_visible (priv->actionbar, !local_only);
-      gtk_list_box_invalidate_filter (GTK_LIST_BOX (priv->listbox));
+      update_places (view);
 
       update_view_mode (view);
 
