@@ -24,6 +24,7 @@
 #include <config.h>
 #include "nautilus-desktop-window.h"
 #include "nautilus-window.h"
+#include "nautilus-application.h"
 
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
@@ -57,7 +58,8 @@ nautilus_desktop_window_update_directory (NautilusDesktopWindow *window)
 
 	window->details->loaded = FALSE;
 	location = g_file_new_for_uri (EEL_DESKTOP_URI);
-	nautilus_window_go_to (NAUTILUS_WINDOW (window), location);
+        nautilus_application_open_location_full (NAUTILUS_APPLICATION (g_application_get_default ()),
+                                                 location, 0, NULL, NAUTILUS_WINDOW (window), NULL);
 	window->details->loaded = TRUE;
 
 	g_object_unref (location);
@@ -85,11 +87,6 @@ nautilus_desktop_window_init_actions (NautilusDesktopWindow *window)
 	action = g_action_map_lookup_action (G_ACTION_MAP (window),
 					     "new-tab");
 	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), FALSE);
-
-	/* Don't allow search on desktop */
-	action = g_action_map_lookup_action (G_ACTION_MAP (window),
-					     "toggle-search");
-	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), FALSE);
 }
 
 static void
@@ -186,29 +183,30 @@ nautilus_desktop_window_constructed (GObject *obj)
 	nautilus_desktop_window_init_selection (window);
 	nautilus_desktop_window_init_actions (window);
 
-	/* Set the accessible name so that it doesn't inherit the cryptic desktop URI. */
-	accessible = gtk_widget_get_accessible (GTK_WIDGET (window));
+	if (window->details->desktop_selection != NULL) {
+		/* Set the accessible name so that it doesn't inherit the cryptic desktop URI. */
+		accessible = gtk_widget_get_accessible (GTK_WIDGET (window));
 
-	if (accessible) {
-		atk_object_set_name (accessible, _("Desktop"));
+		if (accessible) {
+			atk_object_set_name (accessible, _("Desktop"));
+		}
+
+		/* Special sawmill setting */
+		gtk_window_set_wmclass (GTK_WINDOW (window), "desktop_window", "Nautilus");
+
+		/* Point window at the desktop folder.
+		 * Note that nautilus_desktop_window_init is too early to do this.
+		 */
+		nautilus_desktop_window_update_directory (window);
+		gtk_widget_override_background_color (GTK_WIDGET (window), 0, &transparent);
+
+		/* We realize it immediately so that the NAUTILUS_DESKTOP_WINDOW_ID
+		 * property is set so gnome-settings-daemon doesn't try to set
+		 * background. And we do a gdk_flush() to be sure X gets it.
+		 */
+		gtk_widget_realize (GTK_WIDGET (window));
+		gdk_flush ();
 	}
-
-	/* Special sawmill setting */
-	gtk_window_set_wmclass (GTK_WINDOW (window), "desktop_window", "Nautilus");
-
-	/* Point window at the desktop folder.
-	 * Note that nautilus_desktop_window_init is too early to do this.
-	 */
-	nautilus_desktop_window_update_directory (window);
-        gtk_widget_override_background_color (GTK_WIDGET (window), 0, &transparent);
-
-	/* We realize it immediately so that the NAUTILUS_DESKTOP_WINDOW_ID
-	 * property is set so gnome-settings-daemon doesn't try to set
-	 * background. And we do a gdk_flush() to be sure X gets it.
-	 */
-	gtk_widget_realize (GTK_WIDGET (window));
-	gdk_flush ();
-
 }
 
 static void
