@@ -146,7 +146,6 @@ enum {
 };
 
 enum {
-	LOADING_URI,
 	SLOT_ADDED,
 	SLOT_REMOVED,
 	LAST_SIGNAL
@@ -198,13 +197,14 @@ action_go_home (GSimpleAction *action,
 		gpointer       user_data)
 {
 	NautilusWindow *window;
-	NautilusWindowSlot *slot;
+        GFile *home;
 
 	window = NAUTILUS_WINDOW (user_data);
-	slot = nautilus_window_get_active_slot (window);
+        home = g_file_new_for_path (g_get_home_dir ());
 
-	nautilus_window_slot_go_home (slot,
-				      nautilus_event_get_window_open_flags ());
+        nautilus_window_open_location_full (window, home, nautilus_event_get_window_open_flags (), NULL, NULL);
+
+        g_object_unref (home);
 }
 
 static void
@@ -246,7 +246,10 @@ action_up (GSimpleAction *action,
 	if (location != NULL) {
 		parent = g_file_get_parent (location);
 		if (parent != NULL)
-			nautilus_window_slot_go_up (slot, nautilus_event_get_window_open_flags ());
+                        nautilus_window_open_location_full (NAUTILUS_WINDOW (user_data),
+                                                            parent,
+                                                            nautilus_event_get_window_open_flags (),
+                                                            NULL, NULL);
 
 		g_clear_object (&parent);
 	}
@@ -359,19 +362,6 @@ action_go_to_tab (GSimpleAction *action,
 	}
 }
 
-static void
-action_toggle_search (GSimpleAction *action,
-		      GVariant      *state,
-		      gpointer       user_data)
-{
-	NautilusWindowSlot *slot;
-
-	slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (user_data));
-	nautilus_window_slot_set_search_visible (slot, g_variant_get_boolean (state));
-
-	g_simple_action_set_state (action, state);
-}
-
 static void
 action_prompt_for_location_root (GSimpleAction *action,
 				 GVariant      *state,
@@ -434,50 +424,6 @@ action_toggle_state_action_button (GSimpleAction *action,
 	g_variant_unref (current_state);
 }
 
-static void
-action_view_mode (GSimpleAction *action,
-		  GVariant      *value,
-		  gpointer       user_data)
-{
-	const gchar *name;
-	NautilusWindowSlot *slot;
-        GFile *location;
-
-	name =  g_variant_get_string (value, NULL);
-	slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (user_data));
-
-	if (g_strcmp0 (name, "list") == 0) {
-		nautilus_window_slot_set_content_view (slot, NAUTILUS_LIST_VIEW_ID);
-                /* If this change is caused because of the automatic list view
-                 * switch for search, don't set as default list view */
-                location = nautilus_window_slot_get_location (slot);
-                if (!(nautilus_file_is_in_search (nautilus_file_get (location))  &&
-                      g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_LIST_VIEW_ON_SEARCH))) {;
-                        g_settings_set_enum (nautilus_preferences,
-                                             NAUTILUS_PREFERENCES_DEFAULT_FOLDER_VIEWER,
-                                             NAUTILUS_DEFAULT_FOLDER_VIEWER_LIST_VIEW);
-                }
-	} else if (g_strcmp0 (name, "grid") == 0) {
-		nautilus_window_slot_set_content_view (slot, NAUTILUS_CANVAS_VIEW_ID);
-
-                /* If the user manually changed the view mode to grid, disable the automatic
-                 * switch to list view on search */
-                location = nautilus_window_slot_get_location (slot);
-                if (nautilus_file_is_in_search (nautilus_file_get (location))) {
-                      g_settings_set_boolean (nautilus_preferences,
-                                              NAUTILUS_PREFERENCES_LIST_VIEW_ON_SEARCH,
-                                              FALSE);
-                }
-                g_settings_set_enum (nautilus_preferences,
-                                     NAUTILUS_PREFERENCES_DEFAULT_FOLDER_VIEWER,
-                                     NAUTILUS_DEFAULT_FOLDER_VIEWER_ICON_VIEW);
-	} else {
-		g_assert_not_reached ();
-	}
-
-	g_simple_action_set_state (action, value);
-}
-
 static void
 undo_manager_changed (NautilusWindow *window)
 {
@@ -527,13 +473,57 @@ undo_manager_changed (NautilusWindow *window)
 	g_free (redo_description);
 }
 
-void
-nautilus_window_go_to (NautilusWindow *window, GFile *location)
+static void
+on_location_changed (NautilusWindow *window)
 {
-	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
+        gtk_places_sidebar_set_location (GTK_PLACES_SIDEBAR (window->priv->places_sidebar),
+                                         nautilus_window_slot_get_location (nautilus_window_get_active_slot (window)));
+}
 
-	nautilus_window_slot_open_location (nautilus_window_get_active_slot (window),
-					    location, 0);
+static void
+on_slot_loading_changed (NautilusWindowSlot *slot,
+                         GParamSpec         *pspec,
+                         NautilusWindow     *window)
+{
+        if (nautilus_window_get_active_slot (window) == slot)
+                on_location_changed (window);
+}
+
+void
+nautilus_window_open_location_full (NautilusWindow          *window,
+                                    GFile                   *location,
+                                    NautilusWindowOpenFlags  flags,
+                                    GList                   *selection,
+                                    NautilusWindowSlot      *target_slot)
+{
+        NautilusWindowSlot *active_slot;
+        gboolean new_tab_at_end;
+
+        active_slot = nautilus_window_get_active_slot (window);
+
+        /* Assert that we are not managing new windows */
+        g_assert (! (flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW));
+	/* if the flags say we want a new tab, open a slot in the current window */
+	if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0) {
+                new_tab_at_end = g_settings_get_enum (nautilus_preferences, NAUTILUS_PREFERENCES_NEW_TAB_POSITION) == NAUTILUS_NEW_TAB_POSITION_END;
+                if (new_tab_at_end)
+		flags |= NAUTILUS_WINDOW_OPEN_SLOT_APPEND;
+
+		target_slot = nautilus_window_open_slot (window, flags);
+	}
+
+	if (target_slot == NULL) {
+		target_slot = active_slot;
+	}
+
+        /* Make the opened location the one active if we weren't ask for the
+         * oposite, since it's the most usual use case */
+        if (!(flags & NAUTILUS_WINDOW_OPEN_FLAG_DONT_MAKE_ACTIVE)) {
+                gtk_window_present (GTK_WINDOW (window));
+                nautilus_window_set_active_slot (window, target_slot);
+        }
+
+	nautilus_window_slot_open_location_full (target_slot, location, flags, selection);
 }
 
 static int
@@ -626,7 +616,7 @@ location_entry_location_changed_callback (GtkWidget      *widget,
 
 	restore_focus_widget (window);
 
-	nautilus_window_slot_open_location (window->priv->active_slot, location, 0);
+	nautilus_window_open_location_full (window, location, 0, NULL, NULL);
 }
 
 static void
@@ -649,6 +639,21 @@ notebook_switch_page_cb (GtkNotebook    *notebook,
 					 slot);
 }
 
+static void
+connect_slot (NautilusWindow     *window,
+              NautilusWindowSlot *slot)
+{
+        g_signal_connect (slot, "notify::loading",
+                          G_CALLBACK (on_slot_loading_changed), window);
+}
+
+static void
+disconnect_slot (NautilusWindow     *window,
+                 NautilusWindowSlot *slot)
+{
+        g_signal_handlers_disconnect_by_data (slot, window);
+}
+
 static void
 close_slot (NautilusWindow     *window,
 	    NautilusWindowSlot *slot,
@@ -661,6 +666,8 @@ close_slot (NautilusWindow     *window,
 
 	DEBUG ("Closing slot %p", slot);
 
+        disconnect_slot (window, slot);
+
 	window->priv->slots = g_list_remove (window->priv->slots, slot);
 
 	g_signal_emit (window, signals[SLOT_REMOVED], 0, slot);
@@ -678,13 +685,14 @@ close_slot (NautilusWindow     *window,
 
 NautilusWindowSlot *
 nautilus_window_open_slot (NautilusWindow             *window,
-			   NautilusWindowOpenSlotFlags flags)
+			   NautilusWindowOpenFlags flags)
 {
 	NautilusWindowSlot *slot;
 
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
 	slot = nautilus_window_slot_new (window);
+        connect_slot (window, slot);
 
 	g_signal_handlers_block_by_func (window->priv->notebook,
 					 G_CALLBACK (notebook_switch_page_cb),
@@ -709,22 +717,15 @@ void
 nautilus_window_new_tab (NautilusWindow *window)
 {
 	NautilusWindowSlot *current_slot;
-	NautilusWindowSlot *new_slot;
 	NautilusWindowOpenFlags flags;
 	GFile *location;
-	int new_slot_position;
 	char *scheme;
 
 	current_slot = nautilus_window_get_active_slot (window);
 	location = nautilus_window_slot_get_location (current_slot);
 
 	if (location != NULL) {
-		flags = 0;
-
-		new_slot_position = g_settings_get_enum (nautilus_preferences, NAUTILUS_PREFERENCES_NEW_TAB_POSITION);
-		if (new_slot_position == NAUTILUS_NEW_TAB_POSITION_END) {
-			flags = NAUTILUS_WINDOW_OPEN_SLOT_APPEND;
-		}
+		flags = g_settings_get_enum (nautilus_preferences, NAUTILUS_PREFERENCES_NEW_TAB_POSITION);
 
 		scheme = g_file_get_uri_scheme (location);
 		if (strcmp (scheme, "x-nautilus-search") == 0) {
@@ -735,9 +736,8 @@ nautilus_window_new_tab (NautilusWindow *window)
 
 		g_free (scheme);
 
-		new_slot = nautilus_window_open_slot (window, flags);
-		nautilus_window_set_active_slot (window, new_slot);
-		nautilus_window_slot_open_location (new_slot, location, 0);
+                flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
+		nautilus_window_open_location_full (window, location, flags, NULL, NULL);
 		g_object_unref (location);
 	}
 }
@@ -902,34 +902,6 @@ setup_side_pane_width (NautilusWindow *window)
 				window->priv->side_pane_width);
 }
 
-static void
-nautilus_window_location_opened (NautilusWindow *window,
-                                 GFile          *location,
-                                 GError         *error,
-                                 gpointer        user_data)
-{
-	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
-
-	if (error) {
-		NautilusFile *old_file;
-		GFile *old_location;
-
-		old_file = nautilus_window_slot_get_file (window->priv->active_slot);
-		old_location = nautilus_file_get_location (old_file);
-
-		gtk_places_sidebar_set_location (GTK_PLACES_SIDEBAR (window->priv->places_sidebar), old_location);
-
-		g_warning ("error opening location: %s", error->message);
-
-		nautilus_window_slot_display_view_selection_failure (window,
-                                                                     old_file,
-                                                                     location,
-                                                                     error);
-
-		g_object_unref (old_location);
-	}
-}
-
 /* Callback used when the places sidebar changes location; we need to change the displayed folder */
 static void
 open_location_cb (NautilusWindow     *window,
@@ -953,12 +925,8 @@ open_location_cb (NautilusWindow     *window,
 		break;
 	}
 
-	nautilus_window_slot_open_location_full (window->priv->active_slot,
-                                                 location,
-                                                 flags,
-                                                 NULL,
-                                                 (NautilusWindowGoToCallback) nautilus_window_location_opened,
-                                                 NULL);
+        nautilus_application_open_location_full (NAUTILUS_APPLICATION (g_application_get_default ()),
+                                                 location, flags, NULL, NULL, NULL);
 }
 
 /* Callback used when the places sidebar needs us to present an error message */
@@ -1080,21 +1048,6 @@ places_sidebar_drag_perform_drop_cb (GtkPlacesSidebar *sidebar,
 	g_list_free_full (source_uri_list, g_free);
 }
 
-/* Callback for our own loading_uri signal.  We update the sidebar's path. */
-static void
-window_loading_uri_cb (NautilusWindow *window,
-		       char           *location,
-		       gpointer        user_data)
-{
-	if (window->priv->places_sidebar) {
-		GFile *file;
-
-		file = g_file_new_for_uri (location);
-		gtk_places_sidebar_set_location (GTK_PLACES_SIDEBAR (window->priv->places_sidebar), file);
-		g_object_unref (file);
-	}
-}
-
 /* Callback used in the "empty trash" menu item from the places sidebar */
 static void
 action_empty_trash (GSimpleAction *action,
@@ -1283,9 +1236,6 @@ nautilus_window_set_up_sidebar (NautilusWindow *window)
 			  G_CALLBACK (places_sidebar_drag_perform_drop_cb), window);
 	g_signal_connect (window->priv->places_sidebar, "populate-popup",
 			  G_CALLBACK (places_sidebar_populate_popup_cb), window);
-
-	g_signal_connect (window, "loading-uri",
-			  G_CALLBACK (window_loading_uri_cb), window);
 }
 
 void
@@ -1559,9 +1509,8 @@ static void
 on_notification_operation_open_clicked (GtkWidget      *notification,
                                         NautilusWindow *window)
 {
-        nautilus_window_slot_open_location (window->priv->active_slot,
-                                            window->priv->folder_to_open,
-                                            0);
+        nautilus_window_open_location_full (window, window->priv->folder_to_open,
+                                            0, NULL, NULL);
         hide_notification_operation (window);
 }
 
@@ -1633,7 +1582,7 @@ path_bar_location_changed_callback (GtkWidget      *widget,
 	if (i >= 0) {
 		nautilus_window_back_or_forward (window, TRUE, i, 0);
 	} else {
-		nautilus_window_slot_open_location (slot, location, 0);
+                nautilus_window_open_location_full (window, location, 0, NULL, NULL);
 	}
 }
 
@@ -1934,10 +1883,8 @@ const GActionEntry win_entries[] = {
 	{ "new-tab", action_new_tab },
 	{ "enter-location", action_enter_location },
 	{ "bookmark-current-location", action_bookmark_current_location },
-	{ "toggle-search", NULL, NULL, "false", action_toggle_search },
 	{ "undo", action_undo },
 	{ "redo", action_redo },
-	{ "view-mode", NULL, "s", "''", action_view_mode },
 	/* Only accesible by shorcuts */
 	{ "close-current-view", action_close_current_view },
 	{ "go-home", action_go_home },
@@ -1977,9 +1924,6 @@ nautilus_window_initialize_actions (NautilusWindow *window)
 	nautilus_application_add_accelerator (app, "win.forward", "<alt>Right");
 	nautilus_application_add_accelerator (app, "win.enter-location", "<control>l");
 	nautilus_application_add_accelerator (app, "win.new-tab", "<control>t");
-	nautilus_application_add_accelerator (app, "win.toggle-search", "<control>f");
-	nautilus_application_add_accelerator (app, "win.view-mode('list')", "<control>1");
-	nautilus_application_add_accelerator (app, "win.view-mode('grid')", "<control>2");
 	nautilus_application_add_accelerator (app, "win.close-current-view", "<control>w");
 
         /* Special case reload, since users are used to use two shortcuts instead of one */
@@ -2218,25 +2162,6 @@ nautilus_window_close (NautilusWindow *window)
 	NAUTILUS_WINDOW_CLASS (G_OBJECT_GET_CLASS (window))->close (window);
 }
 
-/* reports location change to window's "loading-uri" clients, i.e.
- * sidebar panels [used when switching tabs]. It will emit the pending
- * location, or the existing location if none is pending.
- */
-static void
-nautilus_window_report_location_change (NautilusWindow *window)
-{
-	NautilusWindowSlot *slot;
-	gchar *uri;
-
-	slot = nautilus_window_get_active_slot (window);
-	uri = nautilus_window_slot_get_current_uri (slot);
-
-	if (uri != NULL) {
-		g_signal_emit (window, signals[LOADING_URI], 0, uri);
-		g_free (uri);
-	}
-}
-
 void
 nautilus_window_set_active_slot (NautilusWindow *window, NautilusWindowSlot *new_slot)
 {
@@ -2244,8 +2169,6 @@ nautilus_window_set_active_slot (NautilusWindow *window, NautilusWindowSlot *new
 
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
-	DEBUG ("Setting new slot %p as active", new_slot);
-
 	if (new_slot) {
 		g_assert ((window == nautilus_window_slot_get_window (new_slot)));
 	}
@@ -2256,6 +2179,8 @@ nautilus_window_set_active_slot (NautilusWindow *window, NautilusWindowSlot *new
 		return;
 	}
 
+	DEBUG ("Setting new slot %p as active, old slot inactive %p", new_slot, old_slot);
+
 	/* make old slot inactive if it exists (may be NULL after init, for example) */
 	if (old_slot != NULL) {
 		/* inform slot & view */
@@ -2266,13 +2191,12 @@ nautilus_window_set_active_slot (NautilusWindow *window, NautilusWindowSlot *new
 
 	/* make new slot active, if it exists */
 	if (new_slot) {
-		/* inform sidebar panels */
-                nautilus_window_report_location_change (window);
-
                 nautilus_toolbar_set_active_slot (NAUTILUS_TOOLBAR (window->priv->toolbar), new_slot);
 
 		/* inform slot & view */
                 nautilus_window_slot_set_active (new_slot, TRUE);
+
+                on_location_changed (window);
 	}
 }
 
@@ -2539,16 +2463,6 @@ nautilus_window_class_init (NautilusWindowClass *class)
 				      FALSE,
 				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
 				      G_PARAM_STATIC_STRINGS);
-
-	signals[LOADING_URI] =
-		g_signal_new ("loading-uri",
-			      G_TYPE_FROM_CLASS (class),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1,
-			      G_TYPE_STRING);
 	signals[SLOT_ADDED] =
 		g_signal_new ("slot-added",
 			      G_TYPE_FROM_CLASS (class),
